#ifndef BUSCAEMLARGURA_H_
#define BUSCAEMLARGURA_H_
#include "../cap3/autoreferencia/Fila.h" // @{\it vide Programa~\ref{c_3.19}}@
#include "listaadj/autoreferencia/Grafo.h" // @{\it vide Programa~\ref{c_7.4}}@
#include <iostream>
#include <stdexcept> 
using std::logic_error;
using std::cout;
using std::endl;
using cap3_autoreferencia::Fila; // @{\it vide Programa~\ref{c_3.19}}@
using cap7_listaadj_autoreferencia::Grafo; // @{\it vide Programa~\ref{c_7.4}@
namespace cap7 {
	class BuscaEmLargura {
  public: 
    static const unsigned char branco;
    static const unsigned char cinza;
    static const unsigned char preto;
  private:
    int *d, *antecessor;
    Grafo *grafo;
	  void visitaBfs (int u, unsigned char *cor) throw (logic_error);
  public: 
	  BuscaEmLargura (Grafo *grafo);
	  void buscaEmLargura () throw (logic_error);
	  int _d (int v) const;
	  int _antecessor (int v) const;  
	  void imprimeCaminho (int origem, int v) const ;
	  ~BuscaEmLargura ();
	};
	const unsigned char BuscaEmLargura::branco = 0;
	const unsigned char BuscaEmLargura::cinza  = 1;
	const unsigned char BuscaEmLargura::preto  = 2;
  void BuscaEmLargura::visitaBfs (int u, unsigned char *cor) 
  throw (logic_error) {
    cor[u] = cinza; this->d[u] = 0;
    Fila<int> fila; 
    fila.enfileira (u);
//    cout << "Visita origem:" << u << " cor: cinza F:" << endl;
//    fila.imprime ();
    while (!fila.vazia ()) {
      int *aux = fila.desenfileira ();  u = *aux; delete aux;
      if (!this->grafo->listaAdjVazia (u)) {
        Grafo::Aresta *a = this->grafo->primeiroListaAdj (u);
        while (a != NULL) {
          int v = a->_v2 ();
          if (cor[v] == branco) {
            cor[v] = cinza; this->d[v] = this->d[u] + 1;
            this->antecessor[v] = u; fila.enfileira (v);
          }
          delete a; a = this->grafo->proxAdj (u);
        }
      }
      cor[u] = preto;
//      cout << "Visita " << u << " dist: " << this->d[u] << " cor: preto F:" << endl;
//      fila.imprime ();
    }
  }
  BuscaEmLargura::BuscaEmLargura (Grafo *grafo) {
    this->grafo = grafo; int n = this->grafo->_numVertices();
    this->d = new int[n]; this->antecessor = new int[n];
  }		  
  void BuscaEmLargura::buscaEmLargura () throw (logic_error) {
    unsigned char *cor = new unsigned char[this->grafo->_numVertices ()]; 
    for (int u = 0; u < grafo->_numVertices (); u++) {
      cor[u] = branco; this->d[u] = INT_MAX; 
      this->antecessor[u] = -1;
    }     
    for (int u = 0; u < grafo->_numVertices (); u++)
      if (cor[u] == branco) this->visitaBfs (u, cor);
		delete [] cor;
  }
  int BuscaEmLargura::_d (int v) const { return this->d[v]; }
  int BuscaEmLargura::_antecessor (int v) const { 
  	return this->antecessor[v]; 
  }  
  void BuscaEmLargura::imprimeCaminho (int origem, int v) const {
    if (origem == v) cout << origem << endl;
    else if (this->antecessor[v] == -1) 
      cout << "Nao existe caminho de " << origem << " ate " << v << endl;
    else {
      imprimeCaminho (origem, this->antecessor[v]);
      cout << v << endl;
    }    
  }
  BuscaEmLargura::~BuscaEmLargura () {
    this->grafo = NULL; delete [] this->d; 
    delete [] this->antecessor;
  }  
}

#endif
